@@ -268,15 +268,10 @@ def test_signif_exception():
         r.signif((1,), 3)
 
 
-def test_with_unpickable_objects():
-    gen = (i ** 2 for i in range(10))
-    with pytest.raises(r.UnpickableObjectError):
-        _ = r.round_object(gen, use_copy=True)
-    _ = r.round_object(gen, use_copy=False)
-
-
 def test_with_callable():
-    f = lambda x: x
+    def f(x):
+        return x
+
     assert callable(r.round_object(f))
 
     def f():
@@ -364,10 +359,10 @@ def test_with_class_instance():
 
     # Note that you cannot create a copy of a class instance's attribute
     # and change it inplace in the instance:
-    _ = r.map_object(lambda x: x * 2, a_copy.x)
+    r.map_object(lambda x: x * 2, a_copy.x)
     assert a_copy.x == 10
 
-    _ = r.ceil_object(a)
+    r.ceil_object(a)
     assert a.x == 11
     assert a.y[:3] == (45, 56, 67)
     assert a.y[3] == {"item1": 457, "item2": 91}
@@ -567,10 +562,18 @@ def test_for_Counter():
 
 
 def test_copy_for_frozenset():
-    x = frozenset([
-        222.222, 333.333, 1.000045, "Shout Bamalama!",
-        222.222, 333.333, 1.000045, "Shout Bamalama"
-    ])
+    x = frozenset(
+        [
+            222.222,
+            333.333,
+            1.000045,
+            "Shout Bamalama!",
+            222.222,
+            333.333,
+            1.000045,
+            "Shout Bamalama",
+        ]
+    )
 
     x_rounded = r.round_object(x, 1, True)
     assert x_rounded is not x
@@ -578,57 +581,71 @@ def test_copy_for_frozenset():
 
 
 def test_no_copy_for_frozenset():
-    x = frozenset([
-        222.222, 333.333, 1.000045, "Shout Bamalama!",
-        222.222, 333.333, 1.000045, "Shout Bamalama"
-    ])
+    x = frozenset(
+        [
+            222.222,
+            333.333,
+            1.000045,
+            "Shout Bamalama!",
+            222.222,
+            333.333,
+            1.000045,
+            "Shout Bamalama",
+        ]
+    )
 
     x_rounded = r.round_object(x, 1, False)
     assert x_rounded is not x
-    assert x_rounded == frozenset({1.0, 'Shout Bamalama!', 333.3, 'Shout Bamalama', 222.2})
+    assert x_rounded == frozenset(
+        {1.0, "Shout Bamalama!", 333.3, "Shout Bamalama", 222.2}
+    )
+
 
 def test_copy_for_bool():
     x = True
     x_rounded = r.round_object(x, use_copy=True)
     assert x_rounded is not x
-    assert x_rounded == True
+    assert x_rounded
 
     y = False
     y_rounded = r.round_object(y, use_copy=True)
     assert y_rounded is not y
-    assert y_rounded == False
+    assert not y_rounded
 
 
 def test_no_copy_for_bool():
     x = True
     x_rounded = r.round_object(x, use_copy=False)
     assert x_rounded is not x
-    assert x_rounded == True
+    assert x_rounded
 
     y = False
     y_rounded = r.round_object(y, use_copy=False)
     assert y_rounded is not y
-    assert y_rounded == False
+    assert not y_rounded
 
 
 def test_copy_for_Decimal():
     from decimal import Decimal
+
     x = Decimal(1) / Decimal(7)
     x_rounded = r.round_object(x, 4, use_copy=True)
     assert x is not x_rounded
-    assert x_rounded == Decimal('0.1429')
+    assert x_rounded == Decimal("0.1429")
 
 
 def test_no_copy_for_Decimal():
     from decimal import Decimal
+
     x = Decimal(1) / Decimal(7)
     x_rounded = r.round_object(x, 4, use_copy=False)
     assert x is not x_rounded
-    assert x_rounded == Decimal('0.1429')
+    assert x_rounded == Decimal("0.1429")
 
 
 def test_copy_for_Fraction():
     from fractions import Fraction
+
     x = Fraction(1, 7)
     x_rounded = r.round_object(x, 4, use_copy=True)
     assert x is not x_rounded
@@ -637,6 +654,7 @@ def test_copy_for_Fraction():
 
 def test_no_copy_for_Fraction():
     from fractions import Fraction
+
     x = Fraction(1, 7)
     x_rounded = r.round_object(x, 4, use_copy=False)
     assert x is not x_rounded
@@ -646,7 +664,7 @@ def test_no_copy_for_Fraction():
 def test_copy_for_None():
     x = None
     x_rounded = r.round_object(x, 4, use_copy=True)
-    assert x is x_rounded # as both are None
+    assert x is x_rounded  # as both are None
 
 
 def test_no_copy_for_None():
@@ -657,16 +675,18 @@ def test_no_copy_for_None():
 
 def test_copy_for_coroutine():
     async def ff():
-        await asyncio.sleep(.05)
+        await asyncio.sleep(0.05)
+
     x_rounded = r.round_object(ff, use_copy=True)
     assert x_rounded is ff
 
 
 def test_no_copy_for_coroutine():
     async def ff():
-        await asyncio.sleep(.05)
+        await asyncio.sleep(0.05)
+
     x_rounded = r.round_object(ff, use_copy=False)
-    assert x_rounded is ff # copies are not created in the case of coroutines
+    assert x_rounded is ff  # copies are not created in the case of coroutines
 
 
 def test_copy_for_BuiltinFunctionType():
@@ -717,19 +737,9 @@ def test_no_copy_for_EllipsisType():
     assert x_rounded is x
 
 
-def test_copy_for_FrameType():
-    """Frames are unpickable, so copies cannot be created.
-    
-    Thus, use_copy=True will not work.
-    """
-    import inspect
-    x = inspect.currentframe()
-    with pytest.raises(r.UnpickableObjectError):
-        r.round_object(x, use_copy=True)
-
-
 def test_no_copy_for_FrameType():
     import inspect
+
     x = inspect.currentframe()
     x_rounded = r.round_object(x, use_copy=False)
     assert x_rounded is x
@@ -737,17 +747,17 @@ def test_no_copy_for_FrameType():
 
 def test_copy_for_MemberDescriptorType():
     class Whatever:
-        __slots__ = ('whatever', )
-    
+        __slots__ = ("whatever",)
+
     x = Whatever.__dict__["whatever"]
     x_rounded = r.round_object(x, use_copy=True)
     assert x_rounded is x
-    
+
 
 def test_no_copy_for_MemberDescriptorType():
     class Whatever:
-        __slots__ = ('whatever', )
-    
+        __slots__ = ("whatever",)
+
     x = Whatever.__dict__["whatever"]
     x_rounded = r.round_object(x, use_copy=False)
     assert x_rounded is x
@@ -755,6 +765,7 @@ def test_no_copy_for_MemberDescriptorType():
 
 def test_copy_for_BuiltinMethodType():
     from datetime import date
+
     x = date.today
     x_rounded = r.round_object(x, use_copy=True)
     assert x_rounded is x
@@ -762,6 +773,7 @@ def test_copy_for_BuiltinMethodType():
 
 def test_no_copy_for_BuiltinMethodType():
     from datetime import date
+
     x = date.today
     x_rounded = r.round_object(x, use_copy=False)
     assert x_rounded is x
@@ -770,6 +782,7 @@ def test_no_copy_for_BuiltinMethodType():
 def test_copy_for_MethodWrapperType():
     class Whatever:
         pass
+
     x = Whatever.__eq__
     x_rounded = r.round_object(x, use_copy=True)
     assert x_rounded is x
@@ -778,6 +791,138 @@ def test_copy_for_MethodWrapperType():
 def test_no_copy_for_MethodWrapperType():
     class Whatever:
         pass
+
     x = Whatever.__eq__
     x_rounded = r.round_object(x, use_copy=False)
     assert x_rounded is x
+
+
+def test_signif_edge_case():
+    assert r.signif(123123123123.0002, 7) == 123123100000
+    assert r.signif_object(123123123123.0002, 7) == 123123100000
+
+
+def test_range():
+    rng = range(8, 12)
+    assert list(r.signif_object(rng, 1)) == [8.0, 9.0, 10, 10]
+    assert list(r.signif_object(rng, 1)) == [8.0, 9.0, 10, 10]
+
+    assert list(r.signif_object(rng, 1, True)) == [8.0, 9.0, 10, 10]
+    assert list(r.signif_object(rng, 1, True)) == [8.0, 9.0, 10, 10]
+
+
+def test_filter():
+    flt = filter(lambda x: x >= 8, range(12))
+    assert list(r.signif_object(flt, 1)) == [8.0, 9.0, 10, 10]
+    assert list(r.signif_object(flt, 1)) == []
+
+    flt = filter(lambda x: x >= 8, range(12))
+    assert list(r.signif_object(flt, 1, True)) == [8.0, 9.0, 10, 10]
+    assert list(r.signif_object(flt, 1)) == [8.0, 9.0, 10, 10]
+
+
+def test_generator():
+    gen = (x for x in range(8, 12))
+    assert list(r.signif_object(gen, 1)) == [8.0, 9.0, 10, 10]
+    assert list(r.signif_object(gen, 1)) == []
+
+    gen = (x for x in range(8, 12))
+    assert list(r.signif_object(gen, 1, True)) == [
+        8.0,
+        9.0,
+        10,
+        10,
+    ]  # for generators use_copy is ignored
+    assert list(r.signif_object(gen, 1)) == []
+
+
+def test_copy_for_map():
+    m = map(lambda i: i / 3, range(10))
+    m_rounded_copy = r.round_object(m, 1, use_copy=True)
+    assert list(m_rounded_copy) == [
+        0.0,
+        0.3,
+        0.7,
+        1.0,
+        1.3,
+        1.7,
+        2.0,
+        2.3,
+        2.7,
+        3.0,
+    ]
+    assert len(list(m)) == 10
+
+
+def test_no_copy_for_map_without_copy():
+    m = map(lambda i: i / 3, range(10))
+    m_rounded_no_copy = r.round_object(m, 1, use_copy=False)
+    assert list(m_rounded_no_copy) == [
+        0.0,
+        0.3,
+        0.7,
+        1.0,
+        1.3,
+        1.7,
+        2.0,
+        2.3,
+        2.7,
+        3.0,
+    ]
+    assert len(list(m)) == 0
+
+
+def test_copy_complex_object_with_map():
+    obj = {
+        "list": [1.122, 55.55554, 234.3],
+        "float": 1.11122156,
+        "int": 1212,
+        "dict": {
+            "filter": filter(lambda i: i % 2 != 0, [1, 1.2323, 2, 22.22]),
+            "map": map(lambda i: i / 3, range(10)),
+        },
+    }
+    obj_rounded = r.round_object(obj, 1, use_copy=True)
+    # filter test changed !
+    assert list(obj_rounded["dict"]["filter"]) == [1, 1.2, 22.2]
+    assert list(obj_rounded["dict"]["map"]) == [
+        0.0,
+        0.3,
+        0.7,
+        1.0,
+        1.3,
+        1.7,
+        2.0,
+        2.3,
+        2.7,
+        3.0,
+    ]
+    assert len(list(obj["dict"]["map"])) == 10
+
+
+def test_no_copy_complex_object_with_map():
+    obj = {
+        "list": [1.122, 55.55554, 234.3],
+        "float": 1.11122156,
+        "int": 1212,
+        "dict": {
+            "filter": filter(lambda i: i % 2 != 0, [1, 1.2323, 2, 22.22]),
+            "map": map(lambda i: i / 3, range(10)),
+        },
+    }
+    obj_rounded = r.round_object(obj, 1, use_copy=False)
+
+    assert list(obj_rounded["dict"]["filter"]) == [1, 1.2, 22.2]
+    assert list(obj_rounded["dict"]["map"]) == [
+        0.0,
+        0.3,
+        0.7,
+        1.0,
+        1.3,
+        1.7,
+        2.0,
+        2.3,
+        2.7,
+        3.0,
+    ]
+    assert len(list(obj["dict"]["map"])) == 0
